@@ -18,17 +18,13 @@ gaussian_approx.ngssm<- function(object, max_iter = 100, conv_tol = 1e-8, ...) {
     pmatch(object$distribution, c("poisson", "binomial", "negative binomial"))
   out <- 
     gaussian_approx_model(object, object$initial_mode, max_iter, conv_tol, model_type = 1L)
-  if(ncol(object$xreg) > 0) {
-    beta <- 
-      object$priors[(length(object$priors) - ncol(object$xreg) + 1):length(object$priors)]
-    class(beta) <- "bssm_prior_list"
-  } else beta <- NULL
-  
-  gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, 
-    a1 = object$a1, P1 = object$P1, xreg = if(ncol(object$xreg) > 0) object$xreg, 
-    beta = beta, obs_intercept = object$obs_intercept, 
-    state_intercept = object$state_intercept, state_names = names(object$a1))
-}
+  model <- gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, a1 = object$a1, P1 = object$P1,
+    obs_intercept = object$obs_intercept, state_intercept = object$state_intercept, 
+    state_names = names(object$a1))
+  model$xreg <- object$xreg
+  model$coefs <- object$coefs
+  model
+  }
 #' @method gaussian_approx ng_bsm
 #' @rdname gaussian_approx
 #' @export
@@ -37,36 +33,24 @@ gaussian_approx.ng_bsm <- function(object, max_iter = 100, conv_tol = 1e-8, ...)
   object$distribution <- pmatch(object$distribution, c("poisson", "binomial", "negative binomial"))
   out <- gaussian_approx_model(object, object$initial_mode, max_iter, conv_tol, model_type = 2L)
   
-  if(ncol(object$xreg) > 0) {
-    beta <- object$priors[(length(object$priors) - ncol(object$xreg) + 1):length(object$priors)]
-    if (ncol(object$xreg)==1) {
-      beta <- beta[[1]]
-      class(beta) <- "bssm_prior"
-    } else {
-      class(beta) <- "bssm_prior_list"
-    }
-  } else beta <- NULL
-  
-  gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, a1 = object$a1, P1 = object$P1,
-    xreg = if(ncol(object$xreg) > 0) object$xreg, 
-    beta = beta,
+  model <- gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, a1 = object$a1, P1 = object$P1,
     obs_intercept = object$obs_intercept, state_intercept = object$state_intercept, 
     state_names = names(object$a1))
+  model$xreg <- object$xreg
+  model$coefs <- object$coefs
+  model
 }
 #' @method gaussian_approx svm
 #' @export
 gaussian_approx.svm <- function(object, max_iter = 100, conv_tol = 1e-8, ...) {
   
   out <- gaussian_approx_model(object, object$initial_mode, max_iter, conv_tol, model_type = 3L)
-  if(ncol(object$xreg) > 0) {
-    beta <- object$priors[(length(object$priors) - ncol(object$xreg) + 1):length(object$priors)]
-    class(beta) <- "bssm_prior_list"
-  } else beta <- NULL
-  gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, a1 = object$a1, P1 = object$P1,
-    xreg = if(ncol(object$xreg) > 0) object$xreg, 
-    beta = beta,
+  model <- gssm(y = out$y, Z = object$Z, H = out$H, T = object$T, R = object$R, a1 = object$a1, P1 = object$P1,
     obs_intercept = object$obs_intercept, state_intercept = object$state_intercept, 
     state_names = names(object$a1))
+  model$xreg <- object$xreg
+  model$coefs <- object$coefs
+  model
 }
 
 #' @method gaussian_approx nlg_ssm
