@@ -28,7 +28,7 @@ sim_smoother.gssm <- function(object, nsim = 1,
 
   out <- gaussian_sim_smoother(object, nsim, use_antithetic, seed, model_type = 1L)
   rownames(out) <- names(object$a1)
-  aperm(out, c(2, 1, 3))
+  aperm(out, c(2, 1, 3))[-(length(object$y) + 1), , , drop = FALSE]
 }
 #' @method sim_smoother bsm
 #' @export
@@ -37,7 +37,7 @@ sim_smoother.bsm <- function(object, nsim = 1,
 
   out <- gaussian_sim_smoother(object, nsim, use_antithetic, seed, model_type = 2L)
   rownames(out) <- names(object$a1)
-  aperm(out, c(2, 1, 3))
+  aperm(out, c(2, 1, 3))[-(length(object$y) + 1), , , drop = FALSE]
 }
 #' @method sim_smoother lgg_ssm
 #' @export
@@ -48,10 +48,10 @@ sim_smoother.lgg_ssm <- function(object, nsim = 1,
     object$R, object$a1, object$P1, 
     object$theta, object$obs_intercept, object$state_intercept,
     object$log_prior_pdf, object$known_params, 
-    object$known_tv_params,
+    object$known_tv_params, as.integer(object$time_varying), 
     object$n_states, object$n_etas, nsim, use_antithetic, seed)
   rownames(out) <- names(object$a1)
-  aperm(out, c(2, 1, 3))
+  aperm(out, c(2, 1, 3))[-(length(object$y) + 1), , , drop = FALSE]
 }
 
 #' @method sim_smoother ngssm
