@@ -1,4 +1,4 @@
-#include "mgg_ssm.h"
+
 #include "ugg_ssm.h"
 #include "ung_ssm.h"
 #include "ugg_bsm.h"
@@ -20,14 +20,14 @@ Rcpp::List bsf(const Rcpp::List& model_,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(m, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
   
-  arma::mat at(m, n);
+  arma::mat at(m, n + 1);
   arma::mat att(m, n);
-  arma::cube Pt(m, m, n);
+  arma::cube Pt(m, m, n + 1);
   arma::cube Ptt(m, m, n);
   filter_summary(alpha, at, att, Pt, Ptt, weights);
   
@@ -44,14 +44,14 @@ Rcpp::List bsf(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
       
-      arma::mat at(m, n);
+      arma::mat at(m, n + 1);
       arma::mat att(m, n);
-      arma::cube Pt(m, m, n);
+      arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
       
@@ -71,14 +71,14 @@ Rcpp::List bsf(const Rcpp::List& model_,
     unsigned int m = model.m;
     unsigned n = model.n;
     
-    arma::cube alpha(m, n, nsim_states);
-    arma::mat weights(nsim_states, n);
-    arma::umat indices(nsim_states, n - 1);
+    arma::cube alpha(m, n + 1, nsim_states);
+    arma::mat weights(nsim_states, n + 1);
+    arma::umat indices(nsim_states, n);
     double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
     
-    arma::mat at(m, n);
+    arma::mat at(m, n + 1);
     arma::mat att(m, n);
-    arma::cube Pt(m, m, n);
+    arma::cube Pt(m, m, n + 1);
     arma::cube Ptt(m, m, n);
     filter_summary(alpha, at, att, Pt, Ptt, weights);
     
@@ -95,13 +95,13 @@ Rcpp::List bsf(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
-      arma::mat at(m, n);
+      arma::mat at(m, n + 1);
       arma::mat att(m, n);
-      arma::cube Pt(m, m, n);
+      arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
       
@@ -119,13 +119,13 @@ Rcpp::List bsf(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
-      arma::mat at(m, n);
+      arma::mat at(m, n + 1);
       arma::mat att(m, n);
-      arma::cube Pt(m, m, n);
+      arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
       
@@ -156,16 +156,16 @@ Rcpp::List bsf_smoother(const Rcpp::List& model_,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(m, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
   
-  arma::mat alphahat(model.m, model.n);
-  arma::cube Vt(model.m, model.m, model.n);
+  arma::mat alphahat(model.m, model.n + 1);
+  arma::cube Vt(model.m, model.m, model.n + 1);
   
   filter_smoother(alpha, indices);
-  running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
+  running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
   
   arma::inplace_trans(alphahat);
   return Rcpp::List::create(
@@ -178,15 +178,15 @@ Rcpp::List bsf_smoother(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
-      arma::mat alphahat(model.m, model.n);
-      arma::cube Vt(model.m, model.m, model.n);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
+      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -203,20 +203,17 @@ Rcpp::List bsf_smoother(const Rcpp::List& model_,
     unsigned int m = model.m;
     unsigned n = model.n;
     
-    arma::cube alpha(m, n, nsim_states);
-    arma::mat weights(nsim_states, n);
-    arma::umat indices(nsim_states, n - 1);
+    arma::cube alpha(m, n + 1, nsim_states);
+    arma::mat weights(nsim_states, n + 1);
+    arma::umat indices(nsim_states, n);
     double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
     
-    arma::mat alphahat(model.m, model.n);
-    arma::cube Vt(model.m, model.m, model.n);
+    arma::mat alphahat(model.m, model.n + 1);
+    arma::cube Vt(model.m, model.m, model.n + 1);
     
-    //  if (smoothing_type == 1) {
     filter_smoother(alpha, indices);
-    running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
-    /*} else {
-     Rcpp::stop("Forward-backward smoothing with psi-filter is not yet implemented.");
-    }*/
+    running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+  
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
       Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt, 
@@ -228,19 +225,16 @@ Rcpp::List bsf_smoother(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
-      arma::mat alphahat(model.m, model.n);
-      arma::cube Vt(model.m, model.m, model.n);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
       
-      //  if (smoothing_type == 1) {
       filter_smoother(alpha, indices);
-      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
-      /*} else {
-       Rcpp::stop("Forward-backward smoothing with psi-filter is not yet implemented.");
-      }*/
+      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
         Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt, 
@@ -253,19 +247,16 @@ Rcpp::List bsf_smoother(const Rcpp::List& model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n, nsim_states);
-      arma::mat weights(nsim_states, n);
-      arma::umat indices(nsim_states, n - 1);
+      arma::cube alpha(m, n + 1, nsim_states);
+      arma::mat weights(nsim_states, n + 1);
+      arma::umat indices(nsim_states, n);
       double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
-      arma::mat alphahat(model.m, model.n);
-      arma::cube Vt(model.m, model.m, model.n);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
       
-      //  if (smoothing_type == 1) {
       filter_smoother(alpha, indices);
-      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
-      /*} else {
-       Rcpp::stop("Forward-backward smoothing with psi-filter is not yet implemented.");
-      }*/
+      running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
         Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt, 
@@ -306,9 +297,9 @@ Rcpp::List bsf_nlg(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(m, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
   
   arma::mat at(m, n);
@@ -352,20 +343,17 @@ Rcpp::List bsf_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(m, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, alpha, weights, indices);
   
-  arma::mat alphahat(model.m, model.n);
-  arma::cube Vt(model.m, model.m, model.n);
+  arma::mat alphahat(model.m, model.n + 1);
+  arma::cube Vt(model.m, model.m, model.n + 1);
   
-  //  if (smoothing_type == 1) {
   filter_smoother(alpha, indices);
-  running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n - 1));
-  /*} else {
-   Rcpp::stop("Forward-backward smoothing with psi-filter is not yet implemented.");
-  }*/
+  running_weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+ 
   arma::inplace_trans(alphahat);
   
   return Rcpp::List::create(
