@@ -48,15 +48,15 @@ Rcpp::List bsf_sde(const arma::vec& y, const double x0,
     *xpfun_diffusion, *xpfun_ddiffusion, *xpfun_prior, *xpfun_obs);
   
   unsigned int n = model.n;
-  arma::cube alpha(1, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(1, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, L, alpha, weights, indices);
   
-  arma::mat at(1, n);
-  arma::mat att(1, n);
-  arma::cube Pt(1, 1, n);
-  arma::cube Ptt(1, 1, n);
+  arma::mat at(1, n + 1);
+  arma::mat att(1, n + 1);
+  arma::cube Pt(1, 1, n + 1);
+  arma::cube Ptt(1, 1, n + 1);
   filter_summary(alpha, at, att, Pt, Ptt, weights);
   
   arma::inplace_trans(at);
@@ -85,16 +85,16 @@ Rcpp::List bsf_smoother_sde(const arma::vec& y, const double x0,
     *xpfun_diffusion, *xpfun_ddiffusion, *xpfun_prior, *xpfun_obs);
   
   unsigned int n = model.n;
-  arma::cube alpha(1, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
+  arma::cube alpha(1, n + 1, nsim_states);
+  arma::mat weights(nsim_states, n + 1);
+  arma::umat indices(nsim_states, n);
   double loglik = model.bsf_filter(nsim_states, L, alpha, weights, indices);
   
-  arma::mat alphahat(1, n);
-  arma::cube Vt(1, 1, n);
+  arma::mat alphahat(1, n + 1);
+  arma::cube Vt(1, 1, n + 1);
   
   filter_smoother(alpha, indices);
-  running_weighted_summary(alpha, alphahat, Vt, weights.col(n - 1));
+  running_weighted_summary(alpha, alphahat, Vt, weights.col(n));
   
   arma::inplace_trans(alphahat);
   
@@ -104,43 +104,6 @@ Rcpp::List bsf_smoother_sde(const arma::vec& y, const double x0,
     Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
 }
 
-// [[Rcpp::export]]
-Rcpp::List coupled_bsf_sde(const arma::vec& y, const double x0, 
-  const bool positive, SEXP drift_pntr, SEXP diffusion_pntr, 
-  SEXP ddiffusion_pntr, SEXP log_prior_pdf_pntr, SEXP log_obs_density_pntr,
-  const arma::vec& theta, const unsigned int nsim_states, 
-  const unsigned int L_c, const unsigned int L_f, const unsigned int seed) {
-  
-  Rcpp::XPtr<funcPtr> xpfun_drift(drift_pntr);
-  Rcpp::XPtr<funcPtr> xpfun_diffusion(diffusion_pntr);
-  Rcpp::XPtr<funcPtr> xpfun_ddiffusion(ddiffusion_pntr);
-  Rcpp::XPtr<prior_funcPtr> xpfun_prior(log_prior_pdf_pntr);
-  Rcpp::XPtr<obs_funcPtr> xpfun_obs(log_obs_density_pntr);
-  
-  sde_ssm model(y, theta, x0, positive, seed, *xpfun_drift,
-    *xpfun_diffusion, *xpfun_ddiffusion, *xpfun_prior, *xpfun_obs);
-  
-  unsigned int n = model.n;
-  arma::cube alpha(1, n, nsim_states);
-  arma::mat weights(nsim_states, n);
-  arma::umat indices(nsim_states, n - 1);
-  double loglik = model.coupled_bsf_filter(nsim_states, L_c, L_f, alpha, weights, indices);
-  
-  arma::mat at(1, n);
-  arma::mat att(1, n);
-  arma::cube Pt(1, 1, n);
-  arma::cube Ptt(1, 1, n);
-  filter_summary(alpha, at, att, Pt, Ptt, weights);
-  
-  arma::inplace_trans(at);
-  arma::inplace_trans(att);
-  return Rcpp::List::create(
-    Rcpp::Named("at") = at, Rcpp::Named("att") = att, 
-    Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt, 
-    Rcpp::Named("weights") = weights,
-    Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-}
-
 // [[Rcpp::export]]
 Rcpp::List sde_pm_mcmc(const arma::vec& y, const double x0, 
   const bool positive, SEXP drift_pntr, SEXP diffusion_pntr, 
@@ -161,7 +124,7 @@ Rcpp::List sde_pm_mcmc(const arma::vec& y, const double x0,
   sde_ssm model(y, theta, x0, positive, seed, *xpfun_drift,
     *xpfun_diffusion, *xpfun_ddiffusion, *xpfun_prior, *xpfun_obs);
   
-  mcmc mcmc_run(arma::uvec(theta.n_elem), arma::mat(1,1), n_iter, n_burnin, 
+  mcmc mcmc_run(n_iter, n_burnin, 
     n_thin, model.n, 1, target_acceptance, gamma, S, true);
   
   mcmc_run.pm_mcmc_bsf_sde(model, end_ram, nsim_states, L);
@@ -179,7 +142,7 @@ Rcpp::List sde_da_mcmc(const arma::vec& y, const double x0,
   const bool positive, SEXP drift_pntr, SEXP diffusion_pntr, 
   SEXP ddiffusion_pntr, SEXP log_prior_pdf_pntr, SEXP log_obs_density_pntr,
   const arma::vec& theta, const unsigned int nsim_states, 
-  const unsigned int L_c, const unsigned int L_f, bool coupled, const unsigned int seed, 
+  const unsigned int L_c, const unsigned int L_f, const unsigned int seed, 
   const unsigned int n_iter, 
   const unsigned int n_burnin, const unsigned int n_thin,
   const double gamma, const double target_acceptance, const arma::mat S,
@@ -194,10 +157,10 @@ Rcpp::List sde_da_mcmc(const arma::vec& y, const double x0,
   sde_ssm model(y, theta, x0, positive, seed, *xpfun_drift,
     *xpfun_diffusion, *xpfun_ddiffusion, *xpfun_prior, *xpfun_obs);
   
-  mcmc mcmc_run(arma::uvec(theta.n_elem), arma::mat(1,1), n_iter, n_burnin, 
+  mcmc mcmc_run(n_iter, n_burnin, 
     n_thin, model.n, 1, target_acceptance, gamma, S, true);
   
-  mcmc_run.da_mcmc_bsf_sde(model, end_ram, nsim_states, L_c, L_f, coupled);
+  mcmc_run.da_mcmc_bsf_sde(model, end_ram, nsim_states, L_c, L_f);
   
   return Rcpp::List::create(Rcpp::Named("alpha") = mcmc_run.alpha_storage,
     Rcpp::Named("theta") = mcmc_run.theta_storage.t(),
@@ -212,7 +175,7 @@ Rcpp::List sde_is_mcmc(const arma::vec& y, const double x0,
   const bool positive, SEXP drift_pntr, SEXP diffusion_pntr, 
   SEXP ddiffusion_pntr, SEXP log_prior_pdf_pntr, SEXP log_obs_density_pntr,
   const arma::vec& theta, const unsigned int nsim_states, 
-  const unsigned int L_c, const unsigned int L_f, bool coupled, const unsigned int seed, 
+  const unsigned int L_c, const unsigned int L_f, const unsigned int seed, 
   const unsigned int n_iter, 
   const unsigned int n_burnin, const unsigned int n_thin,
   const double gamma, const double target_acceptance, const arma::mat S,
@@ -235,9 +198,7 @@ Rcpp::List sde_is_mcmc(const arma::vec& y, const double x0,
     mcmc_run.expand();
   }
   
-  
-  mcmc_run.is_correction_bsf(model, nsim_states, L_c, L_f, coupled, 
-    is_type, n_threads);
+  mcmc_run.is_correction_bsf(model, nsim_states, L_c, L_f, is_type, n_threads);
   
   return Rcpp::List::create(Rcpp::Named("alpha") = mcmc_run.alpha_storage,
     Rcpp::Named("theta") = mcmc_run.theta_storage.t(),
