@@ -18,8 +18,8 @@ Rcpp::List gaussian_smoother(const Rcpp::List& model_, const int model_type) {
     n = y.n_rows;
   }
   
-  arma::mat alphahat(m, n);
-  arma::cube Vt(m, m, n);
+  arma::mat alphahat(m, n + 1);
+  arma::cube Vt(m, m, n + 1);
   
   switch (model_type) {
   case -1: {
@@ -49,7 +49,7 @@ Rcpp::List general_gaussian_smoother(const arma::mat& y, SEXP Z, SEXP H,
   const arma::vec& theta, 
   SEXP D, SEXP C,
   SEXP log_prior_pdf, const arma::vec& known_params, 
-  const arma::mat& known_tv_params,
+  const arma::mat& known_tv_params, const arma::uvec& time_varying,
   const unsigned int n_states, const unsigned int n_etas) {
   
   Rcpp::XPtr<lmat_fnPtr> xpfun_Z(Z);
@@ -63,15 +63,15 @@ Rcpp::List general_gaussian_smoother(const arma::mat& y, SEXP Z, SEXP H,
   Rcpp::XPtr<prior_fnPtr> xpfun_prior(log_prior_pdf);
   
   lgg_ssm model(y, *xpfun_Z, *xpfun_H, *xpfun_T, *xpfun_R, *xpfun_a1, *xpfun_P1, 
-    *xpfun_D, *xpfun_C, theta, *xpfun_prior, known_params, known_tv_params, n_states, n_etas,
-    1);
+    *xpfun_D, *xpfun_C, theta, *xpfun_prior, known_params, known_tv_params, 
+    time_varying, n_states, n_etas, 1);
   mgg_ssm mgg_model = model.build_mgg();
   
   unsigned int m = model.m;
   unsigned int n = model.n;
   
-  arma::mat alphahat(m, n);
-  arma::cube Vt(m, m, n);
+  arma::mat alphahat(m, n + 1);
+  arma::cube Vt(m, m, n + 1);
   
   mgg_model.smoother(alphahat, Vt);
   
@@ -97,9 +97,9 @@ Rcpp::List gaussian_ccov_smoother(const Rcpp::List& model_, const int model_type
     n = y.n_rows;
   }
   
-  arma::mat alphahat(m, n);
-  arma::cube Vt(m, m, n);
-  arma::cube Ct(m, m, n);
+  arma::mat alphahat(m, n + 1);
+  arma::cube Vt(m, m, n + 1);
+  arma::cube Ct(m, m, n + 1);
   switch (model_type) {
   case -1: {
     mgg_ssm model(clone(model_), 1);
@@ -173,7 +173,7 @@ arma::cube general_gaussian_sim_smoother(const arma::mat& y, SEXP Z, SEXP H,
   const arma::vec& theta, 
   SEXP D, SEXP C,
   SEXP log_prior_pdf, const arma::vec& known_params, 
-  const arma::mat& known_tv_params,
+  const arma::mat& known_tv_params, const arma::uvec& time_varying,
   const unsigned int n_states, const unsigned int n_etas, const unsigned int nsim, 
   bool use_antithetic, const unsigned int seed) {
   
@@ -188,8 +188,8 @@ arma::cube general_gaussian_sim_smoother(const arma::mat& y, SEXP Z, SEXP H,
   Rcpp::XPtr<prior_fnPtr> xpfun_prior(log_prior_pdf);
   
   lgg_ssm model(y, *xpfun_Z, *xpfun_H, *xpfun_T, *xpfun_R, *xpfun_a1, *xpfun_P1, 
-    *xpfun_D, *xpfun_C, theta, *xpfun_prior, known_params, known_tv_params, n_states, n_etas,
-    seed);
+    *xpfun_D, *xpfun_C, theta, *xpfun_prior, known_params, known_tv_params, 
+    time_varying, n_states, n_etas, seed);
   mgg_ssm mgg_model = model.build_mgg();
   
   return mgg_model.simulate_states();
