@@ -9,8 +9,7 @@ class nlg_amcmc: public mcmc {
 public:
   
   // constructor
-  nlg_amcmc(const arma::uvec& prior_distributions, const arma::mat& prior_parameters,
-    const unsigned int n_iter, const unsigned int n_burnin, const unsigned int n_thin, 
+  nlg_amcmc(const unsigned int n_iter, const unsigned int n_burnin, const unsigned int n_thin, 
     const unsigned int n, const unsigned int m, const double target_acceptance, 
     const double gamma, const arma::mat& S, const bool store_modes);
   
