@@ -11,9 +11,9 @@ public:
 
   
   // update model given the parameters theta
-  void set_theta(const arma::vec& theta);
-  // extract theta from the model
-  arma::vec get_theta() const;
+  void update_model(const arma::vec& new_theta);
+  double log_prior_pdf(const arma::vec& x) const;
+  double log_proposal_ratio(const arma::vec& new_theta, const arma::vec& old_theta) const;
   
 private:
   unsigned int svm_type;
